@@ -514,7 +514,7 @@ var restoreClass = function (readOnly) {
 
 Ext.override(Ext.form.Field, {
     setReadOnly: function (readOnly) {
-        if(!(this.ownerCt && this.ownerCt.readOnly)){
+        if(readOnly || !(this.ownerCt && this.ownerCt.readOnly)){
             setReadOnlyField.call(this, readOnly);
             restoreClass.call(this, readOnly);
         }
@@ -524,7 +524,7 @@ Ext.override(Ext.form.Field, {
 var setReadOnlyTriggerField = Ext.form.TriggerField.prototype.setReadOnly;
 Ext.override(Ext.form.TriggerField, {
     setReadOnly: function (readOnly) {
-        if(!(this.ownerCt && this.ownerCt.readOnly)){
+        if(readOnly || !(this.ownerCt && this.ownerCt.readOnly)){
             setReadOnlyTriggerField.call(this, readOnly);
             restoreClass.call(this, readOnly);
         }
