@@ -1,14 +1,9 @@
-#coding:utf-8
-"""
-Created on 27.02.2010
+# coding: utf-8
+from __future__ import absolute_import
 
-@author: akvarats
-@author: prefer <telepenin@bars-open.ru>
-"""
 from m3_ext.ui.base import ExtUIComponent
-from m3_ext.ui.controls import ExtButton
 
-from base import BaseExtContainer
+from .base import BaseExtContainer
 
 
 #==============================================================================
@@ -273,6 +268,15 @@ class ExtPagingBar(BaseExtContainer):
         return 'new %s' % res if not self._is_function_render else res
 
 
+class ExtCountFreePagingBar(ExtPagingBar):
+    """ Пагинатор не требующий общего количества записей """
+
+    def __init__(self, *args, **kwargs):
+        super(ExtCountFreePagingBar, self).__init__(*args, **kwargs)
+        self._ext_name = 'Ext.m3.CountFreePagingToolbar'
+        self.display_message = u'Показано записей {0} - {1}'
+
+
 class ExtToolbarMenu(ExtUIComponent):
     """
     Класс, позволяющий легко вставлять меню в ToolBar
