@@ -1,19 +1,15 @@
-#coding: utf-8
-"""
-Created on 27.02.2010
+# coding: utf-8
+from __future__ import absolute_import
 
-@author: akvarats
-@author: prefer
-"""
-
-from datetime import datetime, date
+from datetime import date
+from datetime import datetime
 
 from django.conf import settings
-from django.utils.html import escapejs
-
 from m3 import date2str
 
-from base import BaseExtField, BaseExtTriggerField
+from .base import BaseExtField
+from .base import BaseExtTriggerField
+import six
 
 
 #==============================================================================
@@ -41,17 +37,29 @@ class ExtStringField(BaseExtField):
         # форматирует строку при вводе
         self.input_mask = None
 
+        # префикс поля, добавляет к строке
+        # неудаляемые символы при вводе
+        # НЕ РАБОТАЕТ совместно с маской ввода
+        self.prefix = None
+
         self.init_component(*args, **kwargs)
 
     def render_base_config(self):
-        if self.input_mask:
+        if self.prefix:
+            unmask_len = self.max_length - len(self.prefix)
+            prefix = self.prefix + 'A'*unmask_len
+            self.plugins.append("new Ext.m3.InputTextMask('%s')" % prefix)
+        elif self.input_mask:
             self.plugins.append("new Ext.ux.Mask('%s')" % self.input_mask)
 
         # Экранирование значений с обратным слешем
         # Кавычки, апострафы, символы переноса строки и т.д. отрежутся функцией normalize в helpers/__init__.py
         # TODO нужно разобраться почему иногда по-умолчанию приходит None, вместо пустой строки.
         if self.value:
-            self.value = self.value.replace('\\', '\\\\')
+            if not isinstance(self.value, six.string_types):
+                raise RuntimeError(
+                    'Field "{0}" should have string value, but gets {1} "{2}"'
+                    ''.format(self.name, type(self.value), repr(self.value)))
 
         super(ExtStringField, self).render_base_config()
         self._put_config_value('inputType', self.input_type)
@@ -90,9 +98,17 @@ class ExtDateField(BaseExtField):
         # Разрешает перехват нажатий клавиш
         self.enable_key_events = False
 
+        # Если превышена максимальная дата,
+        # то будет отображаться этот текст
+        self.max_text = None
+
         # Максимальная возможная дата
         self.max_value = None
 
+        # Если превышена минимальная дата,
+        # то будет отображаться этот текст
+        self.min_text = None
+
         # Минимально возможная дата
         self.min_value = None
 
@@ -121,6 +137,8 @@ class ExtDateField(BaseExtField):
         self._put_config_value('startDay', self.start_day)
         self._put_config_value('maxValue', self.max_value)
         self._put_config_value('minValue', self.min_value)
+        self._put_config_value('maxText', self.max_text)
+        self._put_config_value('minText', self.min_text)
         self._put_config_value('editable', self.editable)
 
     def render_params(self):
@@ -144,6 +162,10 @@ class ExtNumberField(BaseExtField):
     Поле ввода числового значения
     """
 
+    # Невидимый пробел для того чтобы, можно было убрать разделитель
+    # тысячной и дробной части
+    INVISIBLE_WHITE_SPACE = u'\u200b'
+
     def __init__(self, *args, **kwargs):
         super(ExtNumberField, self).__init__(*args, **kwargs)
 
@@ -151,6 +173,12 @@ class ExtNumberField(BaseExtField):
         # Разделитель целой и дробной части
         self.decimal_separator = None
 
+        # Включение разбиения числа на тысячные разряды
+        self.use_thousand_separator = True
+
+        # Разделитель тысячных частей
+        self.thousand_separator = None
+
         # Признак, что может быть дробная часть
         self.allow_decimals = True
 
@@ -163,13 +191,15 @@ class ExtNumberField(BaseExtField):
         # Маскимально возможное значение
         self.max_value = None
 
-        # Если превышено максимально возможное значение, то будет отображаться этот текст
+        # Если превышено максимально возможное значение,
+        # то будет отображаться этот текст
         self.max_text = None
 
         # Минимальное возможное значение
         self.min_value = None
 
-        # Если превышено минимальное возможное значение, то будет отображаться этот текст
+        # Если превышено минимальное возможное значение,
+        # то будет отображаться этот текст
         self.min_text = None
 
         # Разрешает перехват нажатий клавиш
@@ -183,6 +213,9 @@ class ExtNumberField(BaseExtField):
     def render_base_config(self):
         super(ExtNumberField, self).render_base_config()
         self._put_config_value('decimalSeparator', self.decimal_separator)
+        self._put_config_value(
+            'useThousandSeparator', self.use_thousand_separator)
+        self._put_config_value('thousandSeparator', self.thousand_separator)
         self._put_config_value('allowDecimals', self.allow_decimals)
         self._put_config_value('allowNegative', self.allow_negative)
         self._put_config_value('decimalPrecision', self.decimal_precision)
@@ -244,8 +277,6 @@ class ExtTextArea(BaseExtField):
         return '/%s/' % self.mask_re
 
     def render_base_config(self):
-        if self.value:
-            self.value = escapejs(self.value)
         super(ExtTextArea, self).render_base_config()
         self._put_config_value('maskRe', self.t_render_mask_re, self.mask_re)
 
@@ -267,9 +298,6 @@ class ExtCheckBox(BaseExtField):
     def __init__(self, *args, **kwargs):
         super(ExtCheckBox, self).__init__(*args, **kwargs)
 
-        #TODO: Необходимо отрефакторить под внутриклассовый рендеринг
-        self.template = 'ext-fields/ext-checkbox.js'
-
         # Признак того, что значение выбрано
         self.checked = False
 
@@ -278,6 +306,18 @@ class ExtCheckBox(BaseExtField):
 
         self.init_component(*args, **kwargs)
 
+    def render_base_config(self):
+        super(ExtCheckBox, self).render_base_config()
+
+        self._put_config_value('checked', self.checked, self.checked)
+        self._put_config_value('boxLabel', self.box_label, self.box_label)
+
+    def render(self):
+        self.render_base_config()
+
+        base_config = self._get_config_str()
+        return 'new Ext.form.Checkbox({%s})' % base_config
+
     @property
     def handler_check(self):
         return self._listeners.get('check')
@@ -388,6 +428,10 @@ class ExtHTMLEditor(BaseExtField):
         self.init_component(*args, **kwargs)
 
     def render_base_config(self):
+        if self.value:
+            if not isinstance(self.value, six.string_types):
+                # Если value не строка, то пытаемся привести к unicode
+                self.value = six.text_type(self.value)
         super(ExtHTMLEditor, self).render_base_config()
 
     def render(self):
@@ -406,6 +450,9 @@ class ExtDisplayField(BaseExtField):
     """
     def __init__(self, *args, **kwargs):
         super(ExtDisplayField, self).__init__(*args, **kwargs)
+        # Флаг для экранирования значения
+        self.html_encode = False
+
         self.init_component(*args, **kwargs)
 
     def render_base_config(self):
@@ -416,6 +463,8 @@ class ExtDisplayField(BaseExtField):
         if self.read_only:
             self._set_config_value('cls', None)
 
+        self._put_config_value('htmlEncode', self.html_encode)
+
     def render(self):
         try:
             self.render_base_config()
